Package: ASpli
Type: Package
Title: Analysis of Alternative Splicing Using RNA-Seq
Version: 2.21.0
Date: 2024-03-22
Author: Estefania Mancini, Andres Rabinovich, Javier Iserte, Marcelo Yanovsky and Ariel Chernomoretz
License: GPL
biocViews: ImmunoOncology, GeneExpression, Transcription,
        AlternativeSplicing, Coverage, DifferentialExpression,
        DifferentialSplicing, TimeCourse, RNASeq, GenomeAnnotation,
        Sequencing, Alignment
Depends: methods, grDevices, stats, utils, parallel, edgeR, limma,
        AnnotationDbi
Imports: GenomicRanges, GenomicFeatures, BiocGenerics, IRanges,
        GenomicAlignments, Gviz, S4Vectors, Rsamtools, BiocStyle,
        igraph, htmltools, data.table, UpSetR, tidyr, DT, MASS, grid,
        graphics, pbmcapply, txdbmaker
Description: Integrative pipeline for the analysis of alternative
             splicing using RNAseq.
Suggests:
Maintainer: Ariel Chernomoretz <algo107@gmail.com>
git_url: https://git.bioconductor.org/packages/ASpli
git_branch: devel
git_last_commit: 6fd6add
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 21:15:23 UTC; biocbuild
